require 'faraday'

module Processmaker
  class Client

    attr_reader :access_token

    class << self
      def resources
        {
          cases: CaseResource,
          projects: ProjectResource,
          users: UserResource,
        }
      end
    end

    self.resources.each do |k, v|
      define_method(k) { v.new(connection: connection) }
    end

    def initialize(options = {})
      @access_token = options.with_indifferent_access[:access_token]
      @url          = options.with_indifferent_access[:url]
    end

    def connection
      @faraday ||= Faraday.new connection_options do |req|
        req.adapter :net_http
      end
    end

    def resources
      @resources ||= {}
    end

    private

    def connection_options
      {
        url: @url,
        headers: {
          content_type: 'application/json',
          authorization: "Bearer #{access_token}"
        }
      }
    end
  end
end
